#ifndef AUX_HPP
#define AUX_HPP

#include <utility>
#include <algorithm>
#include <cassert>
#include "../config.h"

/**
 * \file aux.hpp
 * \brief Auxiliary macros and functions (mainly optimized set operations)
 * \author Ralf Wimmer
 * \date 2015-16
 */


/**
 * \def val_assert(x)
 * \brief Macro for asserting conditions
 *
 * If the condition is violated the program is aborted.
 * Using valgrind, a stack trace can be obtained (which is not
 * supported by the standard assert macro).
 */

#if defined(HAVE_VALGRIND_VALGRIND_H) && !defined(NDEBUG)
#include <valgrind/valgrind.h>
#  define val_assert(x) \
        do { if (!(x)) { VALGRIND_PRINTF_BACKTRACE("Assertion failed at %s:%i!\n", __FILE__, __LINE__); assert(x); } } while(0)

#  define val_assert_msg(x, y) \
        do { if (!(x)) { VALGRIND_PRINTF_BACKTRACE("Assertion failed at %s:%i!\nReason: %s\n", __FILE__, __LINE__, y); assert(x); } } while(0)
#else
#  define val_assert(x) \
        assert(x)

#  define val_assert_msg(x, y) \
        assert(x && y)
#endif


#endif
